package jalview.viewmodel;

import jalview.api.AlignmentColsCollectionI;
import jalview.api.AlignmentRowsCollectionI;
import jalview.datamodel.AlignmentI;
import jalview.datamodel.HiddenColumns;
import jalview.datamodel.HiddenSequences;
import jalview.datamodel.VisibleColsCollection;
import jalview.datamodel.VisibleRowsCollection;

public class OverviewDimensionsHideHidden extends OverviewDimensions
{
  private ViewportRanges ranges;

  public OverviewDimensionsHideHidden(ViewportRanges vpranges,
          boolean showAnnotationPanel)
  {
    super(vpranges, showAnnotationPanel);
    ranges = vpranges;
    resetAlignmentDims();
  }

  @Override
  public void updateViewportFromMouse(int mousex, int mousey,
          HiddenSequences hiddenSeqs, HiddenColumns hiddenCols)
  {
    resetAlignmentDims();

    int x = mousex;
    int y = mousey;

    if (x < 0)
    {
      x = 0;
    }

    if (y < 0)
    {
      y = 0;
    }

    //
    // Convert x value to residue position
    //

    // need to determine where scrollCol should be, given x
    // to do this also need to know width of viewport, and some hidden column
    // correction

    // convert x to residues - this is an absolute position
    int xAsRes = Math.round((float) x * alwidth / width);

    // get viewport width in residues
    int vpwidth = ranges.getEndRes() - ranges.getStartRes() + 1;

    if (xAsRes + vpwidth > alwidth)
    {
      // went past the end of the alignment, adjust backwards

      // if last position was before the end of the alignment, need to update
      if ((scrollCol + vpwidth - 1) < alwidth)
      {
        xAsRes = alwidth - vpwidth;
      }
      else
      {
        xAsRes = scrollCol;
      }
    }


    //
    // Convert y value to sequence position
    //

    // convert y to residues
    int yAsSeq = Math.round((float) y * alheight / sequencesHeight);

    // get viewport height in sequences
    // add 1 because height includes both endSeq and startSeq
    int vpheight = ranges.getEndSeq() - ranges.getStartSeq() + 1;

    if (yAsSeq + vpheight > alheight)
    {
      // went past the end of the alignment, adjust backwards
      if ((scrollRow + vpheight - 1) < alheight)
      {
        yAsSeq = alheight - vpheight;
      }
      else
      {
        yAsSeq = scrollRow;
      }
    }

    // update scroll values
    scrollCol = xAsRes;
    scrollRow = yAsSeq;

  }

  @Override
  public void setBoxPosition(HiddenSequences hiddenSeqs,
          HiddenColumns hiddenCols)
  {
    // work with visible values of startRes and endRes
    int startRes = ranges.getStartRes();
    int endRes = ranges.getEndRes();

    // work with visible values of startSeq and endSeq
    int startSeq = ranges.getStartSeq();
    int endSeq = ranges.getEndSeq();

    setBoxPosition(startRes, endRes, startSeq, endSeq);
  }

  @Override
  public AlignmentColsCollectionI getColumns(AlignmentI al)
  {
    return new VisibleColsCollection(0,
            ranges.getAbsoluteAlignmentWidth() - 1, al);
  }

  @Override
  public AlignmentRowsCollectionI getRows(AlignmentI al)
  {
    return new VisibleRowsCollection(0,
            ranges.getAbsoluteAlignmentHeight() - 1, al);
  }

  @Override
  protected void resetAlignmentDims()
  {
    alwidth = ranges.getVisibleAlignmentWidth();
    alheight = ranges.getVisibleAlignmentHeight();
  }
}
